@@ -4,9 +4,9 @@ from __future__ import annotations
 import asyncio
 import datetime
 import logging
-from typing import Optional
+from typing import Any, Callable, Optional
 
-import aiohttp
+import aiohttp  # type: ignore
 import requests  # type: ignore
 
 from .const import MAX_AMPS, MIN_AMPS
@@ -53,22 +53,12 @@ class OpenEVSEWebsocket:
         user=None,
         password=None,
     ):
-        """Initialize a OpenEVSEWebsocket instance.
-        Parameters:
-            server (openevse address):
-                A connected instance.
-            callback (Runnable):
-                Called when interesting events occur. Provides arguments:
-                   msgtype (str): Message type or SIGNAL_* constant
-                   data (str): Current STATE_* or websocket payload contents
-                   error (str): Error message if any or None
-        """
+        """Initialize a OpenEVSEWebsocket instance."""
         self.session = aiohttp.ClientSession()
         self.uri = self._get_uri(server)
-        self._user = (user,)
-        self._password = (password,)
+        self._user = user
+        self._password = password
         self.callback = callback
-        self.subscriptions = ["message"]
         self._state = None
         self.failed_attempts = 0
         self._error_reason = None
@@ -89,74 +79,41 @@ class OpenEVSEWebsocket:
     @staticmethod
     def _get_uri(server):
         """Generate the websocket URI."""
-        return server._url("/:/ws").replace("http", "ws")
+        return server[: server.rfind("/")].replace("http", "ws") + "/ws"
 
     async def running(self):
         """Open a persistent websocket connection and act on events."""
         self.state = STATE_STARTING
+        auth = None
+
+        if self._user and self._password:
+            auth = aiohttp.BasicAuth(self._user, self._password)
 
         try:
-            if self._user and self._password:
-                async with self.session.ws_connect(
-                    self.uri,
-                    heartbeat=15,
-                    auth=aiohttp.BasicAuth(self._user, self._password),
-                ) as ws_client:
-                    self.state = STATE_CONNECTED
-                    self.failed_attempts = 0
+            async with self.session.ws_connect(
+                self.uri,
+                heartbeat=15,
+                auth=auth,
+            ) as ws_client:
+                self.state = STATE_CONNECTED
+                self.failed_attempts = 0
 
-                    async for message in ws_client:
-                        if self.state == STATE_STOPPED:
-                            break
+                async for message in ws_client:
+                    if self.state == STATE_STOPPED:
+                        break
 
-                        if message.type == aiohttp.WSMsgType.TEXT:
-                            msg = message.json()
-                            msgtype = msg["type"]
+                    if message.type == aiohttp.WSMsgType.TEXT:
+                        msg = message.json()
+                        msgtype = "data"
+                        self.callback(msgtype, msg, None)
 
-                            if msgtype not in self.subscriptions:
-                                _LOGGER.debug("Ignoring: %s", msg)
-                                continue
+                    elif message.type == aiohttp.WSMsgType.CLOSED:
+                        _LOGGER.warning("Websocket connection closed")
+                        break
 
-                            else:
-                                self.callback(msgtype, msg, None)
-
-                        elif message.type == aiohttp.WSMsgType.CLOSED:
-                            _LOGGER.warning("AIOHTTP websocket connection closed")
-                            break
-
-                        elif message.type == aiohttp.WSMsgType.ERROR:
-                            _LOGGER.error("AIOHTTP websocket error")
-                            break
-            else:
-                async with self.session.ws_connect(
-                    self.uri,
-                    heartbeat=15,
-                ) as ws_client:
-                    self.state = STATE_CONNECTED
-                    self.failed_attempts = 0
-
-                    async for message in ws_client:
-                        if self.state == STATE_STOPPED:
-                            break
-
-                        if message.type == aiohttp.WSMsgType.TEXT:
-                            msg = message.json()
-                            msgtype = msg["type"]
-
-                            if msgtype not in self.subscriptions:
-                                _LOGGER.debug("Ignoring: %s", msg)
-                                continue
-
-                            else:
-                                self.callback(msgtype, msg, None)
-
-                        elif message.type == aiohttp.WSMsgType.CLOSED:
-                            _LOGGER.warning("AIOHTTP websocket connection closed")
-                            break
-
-                        elif message.type == aiohttp.WSMsgType.ERROR:
-                            _LOGGER.error("AIOHTTP websocket error")
-                            break
+                    elif message.type == aiohttp.WSMsgType.ERROR:
+                        _LOGGER.error("Websocket error")
+                        break
 
         except aiohttp.ClientResponseError as error:
             if error.code == 401:
@@ -191,7 +148,7 @@ class OpenEVSEWebsocket:
                 await asyncio.sleep(5)
 
     async def listen(self):
-        """Close the listening websocket."""
+        """Start the listening websocket."""
         self.failed_attempts = 0
         while self.state != STATE_STOPPED:
             await self.running()
@@ -208,57 +165,133 @@ class OpenEVSE:
         """Connect to an OpenEVSE charger equipped with wifi or ethernet."""
         self._user = user
         self._pwd = pwd
-        self._url = f"http://{host}"
-        self._status = None
-        self._config = None
+        self.url = f"http://{host}/"
+        self._status: dict = {}
+        self._config: dict = {}
         self._override = None
+        self._ws_listening = False
+        self.websocket: Optional[OpenEVSEWebsocket] = None
+        self.callback: Optional[Callable] = None
+        self._loop = None
 
-    def send_command(self, command: str) -> tuple | None:
+    async def send_command(self, command: str) -> tuple | None:
         """Send a RAPI command to the charger and parses the response."""
-        url = f"{self._url}/r"
+        auth = None
+        url = f"{self.url}r"
         data = {"json": 1, "rapi": command}
 
+        if self._user and self._pwd:
+            auth = aiohttp.BasicAuth(self._user, self._pwd)
+
         _LOGGER.debug("Posting data: %s to %s", command, url)
-        if self._user is not None:
-            value = requests.post(url, data=data, auth=(self._user, self._pwd))
-        else:
-            value = requests.post(url, data=data)
+        async with aiohttp.ClientSession() as session:
+            async with session.post(url, data=data, auth=auth) as resp:
+                if resp.status == 400:
+                    _LOGGER.debug("JSON error: %s", await resp.text())
+                    raise ParseJSONError
+                if resp.status == 401:
+                    _LOGGER.debug("Authentication error: %s", await resp)
+                    raise AuthenticationError
 
-        if value.status_code == 400:
-            _LOGGER.debug("JSON error: %s", value.text)
-            raise ParseJSONError
-        if value.status_code == 401:
-            _LOGGER.debug("Authentication error: %s", value)
-            raise AuthenticationError
+                value = await resp.json()
 
-        if "ret" not in value.json():
-            return False, ""
-        resp = value.json()
-        return resp["cmd"], resp["ret"]
+                if "ret" not in value:
+                    return False, ""
+                return value["cmd"], value["ret"]
 
-    def update(self) -> None:
+    async def update(self) -> None:
         """Update the values."""
-        urls = [f"{self._url}/status", f"{self._url}/config"]
+        auth = None
+        urls = [f"{self.url}config"]
 
-        for url in urls:
-            _LOGGER.debug("Updating data from %s", url)
-            if self._user is not None:
-                value = requests.get(url, auth=(self._user, self._pwd))
-            else:
-                value = requests.get(url)
+        if self._user and self._pwd:
+            auth = aiohttp.BasicAuth(self._user, self._pwd)
 
-            if value.status_code == 401:
-                _LOGGER.debug("Authentication error: %s", value)
-                raise AuthenticationError
+        if not self._ws_listening:
+            urls = [f"{self.url}status", f"{self.url}config"]
 
-            if "/status" in url:
-                self._status = value.json()
-            else:
-                self._config = value.json()
+        async with aiohttp.ClientSession() as session:
+            for url in urls:
+                _LOGGER.debug("Updating data from %s", url)
+                async with session.get(url, auth=auth) as resp:
+                    if resp.status == 401:
+                        _LOGGER.debug("Authentication error: %s", resp)
+                        raise AuthenticationError
+
+                    if "/status" in url:
+                        self._status = await resp.json()
+                        _LOGGER.debug("Status update: %s", self._status)
+                    else:
+                        self._config = await resp.json()
+                        _LOGGER.debug("Config update: %s", self._config)
+
+        if not self.websocket:
+            # Start Websocket listening
+            self.websocket = OpenEVSEWebsocket(
+                self.url, self._update_status, self._user, self._pwd
+            )
+            if not self._ws_listening:
+                self._start_listening()
+
+    def _start_listening(self):
+        """Start the websocket listener."""
+        try:
+            _LOGGER.debug("Attempting to find running loop...")
+            self._loop = asyncio.get_running_loop()
+        except RuntimeError:
+            self._loop = asyncio.get_event_loop()
+            _LOGGER.debug("Using new event loop...")
+
+        if not self._ws_listening:
+            self._loop.create_task(self.websocket.listen())
+            pending = asyncio.all_tasks()
+            self._ws_listening = True
+            self._loop.run_until_complete(asyncio.gather(*pending))
+
+    def _update_status(self, msgtype, data, error):
+        """Update data from websocket listener."""
+        if msgtype == SIGNAL_CONNECTION_STATE:
+            if data == STATE_CONNECTED:
+                _LOGGER.debug("Websocket to %s successful", self.url)
+                self._ws_listening = True
+            elif data == STATE_DISCONNECTED:
+                _LOGGER.debug(
+                    "Websocket to %s disconnected, retrying",
+                    self.url,
+                )
+                self._ws_listening = False
+            # Stopped websockets without errors are expected during shutdown
+            # and ignored
+            elif data == STATE_STOPPED and error:
+                _LOGGER.error(
+                    "Websocket to %s failed, aborting [Error: %s]",
+                    self.url,
+                    error,
+                )
+                self._ws_listening = False
+
+        elif msgtype == "data":
+            _LOGGER.debug("ws_data: %s", data)
+            self._status.update(data)
+
+            if self.callback is not None:
+                self.callback()
+
+    def ws_disconnect(self) -> None:
+        """Disconnect the websocket listener."""
+        assert self.websocket
+        self.websocket.close()
+        self._ws_listening = False
+
+    @property
+    def ws_state(self) -> Any:
+        """Return the status of the websocket listener."""
+        assert self.websocket
+        return self.websocket.state
 
     def get_override(self) -> None:
         """Get the manual override status."""
-        url = f"{self._url}/overrride"
+        url = f"{self.url}/overrride"
 
         _LOGGER.debug("Geting data from %s", url)
         if self._user is not None:
@@ -282,7 +315,7 @@ class OpenEVSE:
         auto_release: bool = True,
     ) -> str:
         """Set the manual override status."""
-        url = f"{self._url}/overrride"
+        url = f"{self.url}/overrride"
 
         if state not in ["active", "disabled"]:
             raise ValueError
@@ -310,7 +343,7 @@ class OpenEVSE:
 
     def toggle_override(self) -> None:
         """Toggle the manual override status."""
-        url = f"{self._url}/overrride"
+        url = f"{self.url}/overrride"
 
         _LOGGER.debug("Toggling manual override %s", url)
         if self._user is not None:
@@ -328,7 +361,7 @@ class OpenEVSE:
 
     def clear_override(self) -> None:
         """Clear the manual override status."""
-        url = f"{self._url}/overrride"
+        url = f"{self.url}/overrride"
 
         _LOGGER.debug("Clearing manual overrride %s", url)
         if self._user is not None:
